@@ -12,6 +12,8 @@
 #include "audiosettings.h"
 #include "generalsettings.h"
 #include "mousesettings.h"
+#include "mediaplayer2.h"
+#include "mediaplayer2player.h"
 #include "playbacksettings.h"
 #include "playlistsettings.h"
 #include "subtitlessettings.h"
@@ -28,6 +30,7 @@
 #include <QCoreApplication>
 #include <QGuiApplication>
 #include <QDir>
+#include <QDBusConnection>
 #include <QFileInfo>
 #include <QPointer>
 #include <QQmlApplicationEngine>
@@ -37,8 +40,10 @@
 #include <QQuickView>
 #include <QStandardPaths>
 #include <QStyleFactory>
+#include <QStyle>
 #include <QThread>
 #include <QQmlEngine>
+#include <QMimeDatabase>
 
 #include <KAboutApplicationDialog>
 #include <KAboutData>
@@ -59,10 +64,13 @@ static QApplication *createApplication(int &argc, char **argv, const QString &ap
     QApplication::setOrganizationDomain("georgefb.com");
     QApplication::setApplicationDisplayName("Haruna - Video Player");
     QApplication::setApplicationVersion(Application::version());
-    QApplication::setWindowIcon(QIcon::fromTheme("com.georgefb.haruna"));
+    QApplication::setWindowIcon(QIcon::fromTheme("org.kde.haruna"));
 
     QQuickStyle::setStyle(QStringLiteral("org.kde.desktop"));
     QQuickStyle::setFallbackStyle(QStringLiteral("Fusion"));
+    if (GeneralSettings::useBreezeIconTheme()) {
+        QIcon::setThemeName(QStringLiteral("breeze"));
+    }
 
     QApplication *app = new QApplication(argc, argv);
     return app;
@@ -75,22 +83,24 @@ Application::Application(int &argc, char **argv, const QString &applicationName)
     m_config = KSharedConfig::openConfig("georgefb/haruna.conf");
     m_shortcuts = new KConfigGroup(m_config, "Shortcuts");
     m_schemes = new KColorSchemeManager(this);
+    m_systemDefaultStyle = m_app->style()->objectName();
 
-    m_isBreezeStyleAvailable = QStyleFactory::keys().contains(QStringLiteral("Breeze"));
+    // register mpris dbus service
+    QString mspris2Name(QStringLiteral("org.mpris.MediaPlayer2.haruna"));
+    QDBusConnection::sessionBus().registerService(mspris2Name);
+    QDBusConnection::sessionBus().registerObject(QStringLiteral("/org/mpris/MediaPlayer2"), this, QDBusConnection::ExportAdaptors);
+    // org.mpris.MediaPlayer2 mpris2 interface
+    new MediaPlayer2(this);
 
-    if (GeneralSettings::useBreezeIconTheme()) {
-        QIcon::setThemeName("breeze");
+    if (GeneralSettings::guiStyle() != QStringLiteral("System")) {
+        QApplication::setStyle(GeneralSettings::guiStyle());
     }
-    if (GeneralSettings::useBreezeGuiStyle()) {
-        QApplication::setStyle("Breeze");
-    }
-
-    setupWorkerThread();
 
     // Qt sets the locale in the QGuiApplication constructor, but libmpv
     // requires the LC_NUMERIC category to be set to "C", so change it back.
     std::setlocale(LC_NUMERIC, "C");
 
+    setupWorkerThread();
     setupAboutData();
     setupCommandLineParser();
     registerQmlTypes();
@@ -105,6 +115,7 @@ Application::Application(int &argc, char **argv, const QString &applicationName)
     };
     QObject::connect(m_engine, &QQmlApplicationEngine::objectCreated,
                      m_app, onObjectCreated, Qt::QueuedConnection);
+    m_engine->addImportPath("qrc:/qml");
     m_engine->addImageProvider("thumbnail", new ThumbnailImageProvider());
     setupQmlContextProperties();
     m_engine->load(url);
@@ -132,28 +143,89 @@ void Application::setupWorkerThread()
     thread->start();
 }
 
+void Application::setupAboutData()
+{
+    m_aboutData = KAboutData(QStringLiteral("haruna"),
+                             i18n("Haruna Video Player"),
+                             Application::version());
+    m_aboutData.setShortDescription(i18n("A configurable video player."));
+    m_aboutData.setLicense(KAboutLicense::GPL_V3);
+    m_aboutData.setCopyrightStatement(i18n("(c) 2019-2021"));
+    m_aboutData.setHomepage(QStringLiteral("https://github.com/g-fb/haruna"));
+    m_aboutData.setBugAddress(QStringLiteral("https://github.com/g-fb/haruna/issues").toUtf8());
+    m_aboutData.setDesktopFileName("org.kde.haruna");
+
+    m_aboutData.addAuthor(i18n("George Florea Bănuș"),
+                        i18n("Developer"),
+                        QStringLiteral("georgefb899@gmail.com"),
+                        QStringLiteral("https://georgefb.com"));
+
+    KAboutData::setApplicationData(m_aboutData);
+}
+
+void Application::setupCommandLineParser()
+{
+    QCommandLineParser parser;
+    m_aboutData.setupCommandLine(&parser);
+    parser.addPositionalArgument(QStringLiteral("file"), i18n("File to open"));
+    parser.process(*m_app);
+    m_aboutData.processCommandLine(&parser);
+
+    for (auto i = 0; i < parser.positionalArguments().size(); ++i) {
+        addArgument(i, parser.positionalArguments().at(i));
+    }
+}
+
+void Application::registerQmlTypes()
+{
+    qmlRegisterType<MpvObject>("mpv", 1, 0, "MpvObject");
+    qRegisterMetaType<PlayListModel*>();
+    qRegisterMetaType<PlayListItem*>();
+    qRegisterMetaType<QAction*>();
+    qRegisterMetaType<TracksModel*>();
+    qRegisterMetaType<KFileMetaData::PropertyMap>("KFileMetaData::PropertyMap");
+}
+
 void Application::setupQmlSettingsTypes()
 {
     auto audioProvider = [](QQmlEngine *, QJSEngine *) -> QObject * { return AudioSettings::self(); };
-    qmlRegisterSingletonType<AudioSettings>("com.georgefb.haruna", 1, 0, "AudioSettings", audioProvider);
+    qmlRegisterSingletonType<AudioSettings>("org.kde.haruna", 1, 0, "AudioSettings", audioProvider);
 
     auto generalProvider = [](QQmlEngine *, QJSEngine *) -> QObject * { return GeneralSettings::self(); };
-    qmlRegisterSingletonType<GeneralSettings>("com.georgefb.haruna", 1, 0, "GeneralSettings", generalProvider);
+    qmlRegisterSingletonType<GeneralSettings>("org.kde.haruna", 1, 0, "GeneralSettings", generalProvider);
 
     auto mouseProvider = [](QQmlEngine *, QJSEngine *) -> QObject * { return MouseSettings::self(); };
-    qmlRegisterSingletonType<MouseSettings>("com.georgefb.haruna", 1, 0, "MouseSettings", mouseProvider);
+    qmlRegisterSingletonType<MouseSettings>("org.kde.haruna", 1, 0, "MouseSettings", mouseProvider);
 
     auto playbackProvider = [](QQmlEngine *, QJSEngine *) -> QObject * { return PlaybackSettings::self(); };
-    qmlRegisterSingletonType<PlaybackSettings>("com.georgefb.haruna", 1, 0, "PlaybackSettings", playbackProvider);
+    qmlRegisterSingletonType<PlaybackSettings>("org.kde.haruna", 1, 0, "PlaybackSettings", playbackProvider);
 
     auto playlistProvider = [](QQmlEngine *, QJSEngine *) -> QObject * { return PlaylistSettings::self(); };
-    qmlRegisterSingletonType<PlaylistSettings>("com.georgefb.haruna", 1, 0, "PlaylistSettings", playlistProvider);
+    qmlRegisterSingletonType<PlaylistSettings>("org.kde.haruna", 1, 0, "PlaylistSettings", playlistProvider);
 
     auto subtitlesProvider = [](QQmlEngine *, QJSEngine *) -> QObject * { return SubtitlesSettings::self(); };
-    qmlRegisterSingletonType<SubtitlesSettings>("com.georgefb.haruna", 1, 0, "SubtitlesSettings", subtitlesProvider);
+    qmlRegisterSingletonType<SubtitlesSettings>("org.kde.haruna", 1, 0, "SubtitlesSettings", subtitlesProvider);
 
     auto videoProvider = [](QQmlEngine *, QJSEngine *) -> QObject * { return VideoSettings::self(); };
-    qmlRegisterSingletonType<VideoSettings>("com.georgefb.haruna", 1, 0, "VideoSettings", videoProvider);
+    qmlRegisterSingletonType<VideoSettings>("org.kde.haruna", 1, 0, "VideoSettings", videoProvider);
+}
+
+void Application::setupQmlContextProperties()
+{
+    std::unique_ptr<LockManager> lockManager = std::make_unique<LockManager>();
+    std::unique_ptr<SubtitlesFoldersModel> subsFoldersModel = std::make_unique<SubtitlesFoldersModel>();
+
+    m_engine->rootContext()->setContextProperty(QStringLiteral("app"), this);
+    qmlRegisterUncreatableType<Application>("Application", 1, 0, "Application",
+                                            QStringLiteral("Application should not be created in QML"));
+
+    m_engine->rootContext()->setContextProperty(QStringLiteral("mediaPlayer2Player"), new MediaPlayer2Player(this));
+
+    m_engine->rootContext()->setContextProperty(QStringLiteral("lockManager"), lockManager.release());
+    qmlRegisterUncreatableType<LockManager>("LockManager", 1, 0, "LockManager",
+                                            QStringLiteral("LockManager should not be created in QML"));
+
+    m_engine->rootContext()->setContextProperty(QStringLiteral("subsFoldersModel"), subsFoldersModel.release());
 }
 
 QUrl Application::configFilePath()
@@ -177,7 +249,13 @@ QUrl Application::configFolderPath()
 
 QString Application::version()
 {
-    return QStringLiteral("0.5.0");
+    return QStringLiteral("0.6.3");
+}
+
+bool Application::hasYoutubeDl()
+{
+    return !QStandardPaths::findExecutable(QStringLiteral("youtube-dl")).isEmpty();
+
 }
 
 QUrl Application::parentUrl(const QString &path)
@@ -209,25 +287,9 @@ QUrl Application::pathToUrl(const QString &path)
     return url;
 }
 
-void Application::setupQmlContextProperties()
+bool Application::isYoutubePlaylist(const QString &path)
 {
-    std::unique_ptr<LockManager> lockManager = std::make_unique<LockManager>();
-    std::unique_ptr<SubtitlesFoldersModel> subsFoldersModel = std::make_unique<SubtitlesFoldersModel>();
-    std::unique_ptr<PlayListModel> playListModel = std::make_unique<PlayListModel>();
-
-    m_engine->rootContext()->setContextProperty("playListModel", playListModel.release());
-    qmlRegisterUncreatableType<PlayListModel>("PlayListModel", 1, 0, "PlayListModel",
-                                               QStringLiteral("PlayListModel should not be created in QML"));
-
-    m_engine->rootContext()->setContextProperty(QStringLiteral("app"), this);
-    qmlRegisterUncreatableType<Application>("Application", 1, 0, "Application",
-                                            QStringLiteral("Application should not be created in QML"));
-
-    m_engine->rootContext()->setContextProperty(QStringLiteral("lockManager"), lockManager.release());
-    qmlRegisterUncreatableType<LockManager>("LockManager", 1, 0, "LockManager",
-                                            QStringLiteral("LockManager should not be created in QML"));
-
-    m_engine->rootContext()->setContextProperty(QStringLiteral("subsFoldersModel"), subsFoldersModel.release());
+    return path.contains("youtube.com/playlist?list");
 }
 
 QString Application::formatTime(const double time)
@@ -237,11 +299,6 @@ QString Application::formatTime(const double time)
     return formattedTime;
 }
 
-QUrl Application::getPathFromArg(const QString &arg)
-{
-    return QUrl::fromUserInput(arg, QDir::currentPath());
-}
-
 void Application::hideCursor()
 {
     QApplication::setOverrideCursor(Qt::BlankCursor);
@@ -274,7 +331,7 @@ QAction *Application::action(const QString &name)
     return resultAction;
 }
 
-QString Application::getFileContent(QString file)
+QString Application::getFileContent(const QString &file)
 {
     QFile f(file);
     f.open(QIODevice::ReadOnly);
@@ -283,20 +340,32 @@ QString Application::getFileContent(QString file)
     return content;
 }
 
+QString Application::mimeType(const QString &file)
+{
+    QMimeDatabase db;
+    QMimeType mime = db.mimeTypeForFile(file);
+    return mime.name();
+}
+
+QStringList Application::availableGuiStyles()
+{
+    return QStyleFactory::keys();
+}
+
+void Application::setGuiStyle(const QString &style)
+{
+    if (style == "Default") {
+        QApplication::setStyle(m_systemDefaultStyle);
+        return;
+    }
+    QApplication::setStyle(style);
+}
+
 QAbstractItemModel *Application::colorSchemesModel()
 {
     return m_schemes->model();
 }
 
-void Application::registerQmlTypes()
-{
-    qmlRegisterType<MpvObject>("mpv", 1, 0, "MpvObject");
-    qRegisterMetaType<QAction*>();
-    qRegisterMetaType<TracksModel*>();
-    qRegisterMetaType<KFileMetaData::PropertyMap>("KFileMetaData::PropertyMap");
-
-}
-
 void Application::activateColorScheme(const QString &name)
 {
     m_schemes->activateScheme(m_schemes->indexForScheme(name));
@@ -324,38 +393,6 @@ void Application::aboutApplication()
     dialog->show();
 }
 
-void Application::setupAboutData()
-{
-    m_aboutData = KAboutData(QStringLiteral("haruna"),
-                             i18n("Haruna Video Player"),
-                             Application::version());
-    m_aboutData.setShortDescription(i18n("A configurable video player."));
-    m_aboutData.setLicense(KAboutLicense::GPL_V3);
-    m_aboutData.setCopyrightStatement(i18n("(c) 2019-2021"));
-    m_aboutData.setHomepage(QStringLiteral("https://github.com/g-fb/haruna"));
-    m_aboutData.setBugAddress(QStringLiteral("https://github.com/g-fb/haruna/issues").toUtf8());
-
-    m_aboutData.addAuthor(i18n("George Florea Bănuș"),
-                        i18n("Developer"),
-                        QStringLiteral("georgefb899@gmail.com"),
-                        QStringLiteral("https://georgefb.com"));
-
-    KAboutData::setApplicationData(m_aboutData);
-}
-
-void Application::setupCommandLineParser()
-{
-    QCommandLineParser parser;
-    m_aboutData.setupCommandLine(&parser);
-    parser.addPositionalArgument(QStringLiteral("file"), i18n("File to open"));
-    parser.process(*m_app);
-    m_aboutData.processCommandLine(&parser);
-
-    for (auto i = 0; i < parser.positionalArguments().size(); ++i) {
-        addArgument(i, parser.positionalArguments().at(i));
-    }
-}
-
 void Application::setupActions(const QString &actionName)
 {
     if (actionName == QStringLiteral("screenshot")) {
@@ -391,7 +428,7 @@ void Application::setupActions(const QString &actionName)
 
     if (actionName == QStringLiteral("togglePlaylist")) {
         auto action = new HAction();
-        action->setText(i18n("Toggle Playlist"));
+        action->setText(i18n("Playlist"));
         action->setIcon(QIcon::fromTheme("view-media-playlist"));
         m_collection.setDefaultShortcut(action, Qt::Key_P);
         m_collection.addAction(actionName, action);
@@ -788,5 +825,11 @@ void Application::setupActions(const QString &actionName)
         m_collection.setDefaultShortcut(action, Qt::SHIFT + Qt::Key_R);
         m_collection.addAction(actionName, action);
     }
+    if (actionName == QStringLiteral("toggleDeinterlacing")) {
+        auto action = new HAction();
+        action->setText(i18n("Toggle deinterlacing"));
+        m_collection.setDefaultShortcut(action, Qt::Key_D);
+        m_collection.addAction(actionName, action);
+    }
     m_collection.readSettings(m_shortcuts);
 }
