@@ -6,14 +6,23 @@
 
 #include "_debug.h"
 #include "mpvobject.h"
+#include "application.h"
+#include "generalsettings.h"
 #include "playbacksettings.h"
+#include "videosettings.h"
+#include "playlistitem.h"
 #include "track.h"
 #include "tracksmodel.h"
 
+#include <QCryptographicHash>
 #include <QDir>
+#include <QJsonArray>
+#include <QJsonDocument>
+#include <QJsonObject>
 #include <QObject>
 #include <QOpenGLContext>
 #include <QOpenGLFramebufferObject>
+#include <QProcess>
 #include <QQuickWindow>
 #include <QStandardPaths>
 #include <QtGlobal>
@@ -90,6 +99,7 @@ MpvObject::MpvObject(QQuickItem * parent)
     , mpv_gl(nullptr)
     , m_audioTracksModel(new TracksModel)
     , m_subtitleTracksModel(new TracksModel)
+    , m_playlistModel(new PlayListModel)
 {
     if (!mpv)
         throw std::runtime_error("could not create mpv context");
@@ -97,14 +107,11 @@ MpvObject::MpvObject(QQuickItem * parent)
 //    setProperty("terminal", "yes");
 //    setProperty("msg-level", "all=v");
 
-    if (PlaybackSettings::useHWDecoding()) {
-        setProperty("hwdec", "yes");
-    } else {
-        setProperty("hwdec", "no");
-    }
-
-    setProperty("screenshot-template", "%x/screenshots/%n");
+    QString hwdec = PlaybackSettings::useHWDecoding() ? PlaybackSettings::hWDecoding() : "no";
+    setProperty("hwdec", hwdec);
+    setProperty("screenshot-template", VideoSettings::screenshotTemplate());
     setProperty("sub-auto", "exact");
+    setProperty("volume-max", "100");
 
     mpv_observe_property(mpv, 0, "media-title", MPV_FORMAT_STRING);
     mpv_observe_property(mpv, 0, "time-pos", MPV_FORMAT_DOUBLE);
@@ -120,14 +127,7 @@ MpvObject::MpvObject(QQuickItem * parent)
     mpv_observe_property(mpv, 0, "brightness", MPV_FORMAT_INT64);
     mpv_observe_property(mpv, 0, "gamma", MPV_FORMAT_INT64);
     mpv_observe_property(mpv, 0, "saturation", MPV_FORMAT_INT64);
-
-    QString configPath = QStandardPaths::writableLocation(QStandardPaths::ConfigLocation);
-    QString watchLaterPath = configPath.append("/georgefb/watch-later");
-    setProperty("watch-later-directory", watchLaterPath);
-    QDir watchLaterDir(watchLaterPath);
-    if (!watchLaterDir.exists()) {
-        QDir().mkdir(watchLaterPath);
-    }
+    mpv_observe_property(mpv, 0, "track-list", MPV_FORMAT_NODE);
 
     if (mpv_initialize(mpv) < 0)
         throw std::runtime_error("could not initialize mpv context");
@@ -156,6 +156,16 @@ MpvObject::~MpvObject()
     mpv_terminate_destroy(mpv);
 }
 
+PlayListModel *MpvObject::playlistModel()
+{
+    return m_playlistModel;
+}
+
+void MpvObject::setPlaylistModel(PlayListModel *model)
+{
+    m_playlistModel = model;
+}
+
 QString MpvObject::mediaTitle()
 {
     return getProperty("media-title").toString();
@@ -364,6 +374,20 @@ QQuickFramebufferObject::Renderer *MpvObject::createRenderer() const
     return new MpvRenderer(const_cast<MpvObject *>(this));
 }
 
+void MpvObject::loadFile(const QString &file, bool updateLastPlayedFile)
+{
+    setProperty("ytdl-format", PlaybackSettings::ytdlFormat());
+    command(QStringList() << "loadfile" << file);
+
+    if (updateLastPlayedFile) {
+        GeneralSettings::setLastPlayedFile(file);
+        GeneralSettings::self()->save();
+    } else {
+        GeneralSettings::setLastPlaylistIndex(m_playlistModel->getPlayingVideo());
+        GeneralSettings::self()->save();
+    }
+}
+
 void MpvObject::mpvEvents(void *ctx)
 {
     QMetaObject::invokeMethod(static_cast<MpvObject*>(ctx), "eventHandler", Qt::QueuedConnection);
@@ -377,6 +401,10 @@ void MpvObject::eventHandler()
             break;
         }
         switch (event->event_id) {
+        case MPV_EVENT_START_FILE: {
+            emit fileStarted();
+            break;
+        }
         case MPV_EVENT_FILE_LOADED: {
             emit fileLoaded();
             break;
@@ -384,7 +412,9 @@ void MpvObject::eventHandler()
         case MPV_EVENT_END_FILE: {
             auto prop = (mpv_event_end_file *)event->data;
             if (prop->reason == MPV_END_FILE_REASON_EOF) {
-                emit endOfFile();
+                emit endFile("eof");
+            } else if(prop->reason == MPV_END_FILE_REASON_ERROR) {
+                emit endFile("error");
             }
             break;
         }
@@ -426,14 +456,10 @@ void MpvObject::eventHandler()
             } else if (strcmp(prop->name, "sid") == 0) {
                 if (prop->format == MPV_FORMAT_INT64) {
                     emit subtitleIdChanged();
-                } else {
-                    emit subtitleIdChanged();
                 }
             } else if (strcmp(prop->name, "secondary-sid") == 0) {
                 if (prop->format == MPV_FORMAT_INT64) {
                     emit secondarySubtitleIdChanged();
-                } else {
-                    emit secondarySubtitleIdChanged();
                 }
             } else if (strcmp(prop->name, "contrast") == 0) {
                 if (prop->format == MPV_FORMAT_INT64) {
@@ -451,6 +477,10 @@ void MpvObject::eventHandler()
                 if (prop->format == MPV_FORMAT_INT64) {
                     emit saturationChanged();
                 }
+            } else if (strcmp(prop->name, "track-list") == 0) {
+                if (prop->format == MPV_FORMAT_NODE) {
+                    loadTracks();
+                }
             }
             break;
         }
@@ -527,18 +557,127 @@ TracksModel *MpvObject::audioTracksModel() const
     return m_audioTracksModel;
 }
 
-int MpvObject::setProperty(const QString &name, const QVariant &value)
+void MpvObject::getYouTubePlaylist(const QString &path)
 {
-    return mpv::qt::set_property(mpv, name, value);
+    m_playlistModel->clear();
+
+    // use youtube-dl to get the required playlist info as json
+    auto ytdlProcess = new QProcess();
+    ytdlProcess->setProgram("youtube-dl");
+    ytdlProcess->setArguments(QStringList() << "-J" << "--flat-playlist" << path);
+    ytdlProcess->start();
+
+    QObject::connect(ytdlProcess, (void (QProcess::*)(int,QProcess::ExitStatus))&QProcess::finished,
+                     this, [=](int, QProcess::ExitStatus) {
+        // use the json to populate the playlist model
+        using Playlist = QList<PlayListItem*>;
+        Playlist m_playList;
+
+        QString json = ytdlProcess->readAllStandardOutput();
+        QJsonObject obj;
+        QJsonValue entries = QJsonDocument::fromJson(json.toUtf8())["entries"];
+
+        QString playlistFileContent;
+
+        for (int i = 0; i < entries.toArray().size(); ++i) {
+            auto url = QString("https://youtu.be/%1").arg(entries[i]["id"].toString());
+            auto title = entries[i]["title"].toString();
+            auto duration = entries[i]["duration"].toDouble();
+
+            auto video = new PlayListItem(url, i, m_playlistModel);
+            video->setMediaTitle(!title.isEmpty() ? title : url);
+            video->setFileName(!title.isEmpty() ? title : url);
+
+            video->setDuration(Application::formatTime(duration));
+            m_playList.append(video);
+
+            playlistFileContent += QString("%1,%2,%3\n").arg(url, title, QString::number(duration));
+        }
+
+        // save playlist to disk
+        m_playlistModel->setPlayList(m_playList);
+
+        emit youtubePlaylistLoaded();
+    });
 }
 
-QVariant MpvObject::getProperty(const QString &name)
+int MpvObject::setProperty(const QString &name, const QVariant &value, bool debug)
 {
-    auto value = mpv::qt::get_property(mpv, name);
-    return value;
+    auto result = mpv::qt::set_property(mpv, name, value);
+    if (debug) {
+        DEBUG << mpv::qt::get_error(result);
+    }
+    return result;
+}
+
+QVariant MpvObject::getProperty(const QString &name, bool debug)
+{
+    auto result = mpv::qt::get_property(mpv, name);
+    if (debug) {
+        DEBUG << mpv::qt::get_error(result);
+    }
+    return result;
 }
 
 QVariant MpvObject::command(const QVariant &params)
 {
     return mpv::qt::command(mpv, params);
 }
+
+void MpvObject::saveTimePosition()
+{
+    // saving position is disabled
+    if (PlaybackSettings::minDurationToSavePosition() == -1) {
+        return;
+    }
+    // position is saved only for files longer than PlaybackSettings::minDurationToSavePosition()
+    if (getProperty("duration").toInt() < PlaybackSettings::minDurationToSavePosition() * 60) {
+        return;
+    }
+
+    auto hash = md5(getProperty("path").toString());
+    auto timePosition = getProperty("time-pos");
+    auto configPath = QStandardPaths::writableLocation(QStandardPaths::ConfigLocation);
+    KConfig *config = new KConfig(configPath.append("/georgefb/watch-later/").append(hash));
+    config->group("").writeEntry("TimePosition", timePosition);
+    config->sync();
+}
+
+double MpvObject::loadTimePosition()
+{
+    // saving position is disabled
+    if (PlaybackSettings::minDurationToSavePosition() == -1) {
+        return 0;
+    }
+    // position is saved only for files longer than PlaybackSettings::minDurationToSavePosition()
+    // but there can be cases when there is a saved position for files lower than minDurationToSavePosition()
+    // when minDurationToSavePosition() was increased after position was already saved
+    if (getProperty("duration").toInt() < PlaybackSettings::minDurationToSavePosition() * 60) {
+        return 0;
+    }
+
+    auto hash = md5(getProperty("path").toString());
+    auto configPath = QStandardPaths::writableLocation(QStandardPaths::ConfigLocation);
+    KConfig *config = new KConfig(configPath.append("/georgefb/watch-later/").append(hash));
+    int position = config->group("").readEntry("TimePosition", QString::number(0)).toDouble();
+
+    return position;
+}
+
+void MpvObject::resetTimePosition()
+{
+    auto hash = md5(getProperty("path").toString());
+    auto configPath = QStandardPaths::writableLocation(QStandardPaths::ConfigLocation);
+    QFile f(configPath.append("/georgefb/watch-later/").append(hash));
+
+    if (f.exists()) {
+        f.remove();
+    }
+}
+
+QString MpvObject::md5(const QString &str)
+{
+    auto md5 = QCryptographicHash::hash((str.toUtf8()), QCryptographicHash::Md5);
+
+    return QString(md5.toHex());
+}
